/* labels.h
 * Manage the labelling system.  These are used to provide the necessary
 * interlinking of branches and subroutine calls.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>
 */

#ifndef __label_h
#define __label_h

#define	Lnull		0x00	/* Unused label */

#define	Ltypemask	0x0F	/* Label type mask */
#define	Llong		0x01	/* Label is 32 bits long */
#define	Llong30		0x02	/* Label is 30 bits long (SPARC) */
#define	Llong22		0x03	/* Label is 22 bits long (SPARC) */
#define	Llong22x10	0x04	/* Label is split into 22 and 10 bits (SPARC) */

#define	Lgeneral	0x00	/* Label reference general code */
#define	Lcode		0x10	/* Label references bytecode offset */

#define	Lexternal	0x00	/* Label references external routine */
#define	Linternal	0x20	/* Label references internal routine */

#define	Labsolute	0x00	/* Absolute value */
#define	Lrelative	0x40	/* Relative value to place of insertion */


typedef struct _label_ {
	uintp		to;
	uintp		at;
	uintp		from;
	uint8		type;
	struct _label_*	next;
	union {
		double	dval;
		uint64	lval;
		uint32	word[2];
	}		u;
	uint8		size;
	struct _label_*	nextconst;
} label;

#define	ALLOCLABELNR	32

label* nextLabel(void);

#endif
