/* ops.h
 *
 * Opcode operand lengths and control flow status.
 * Ideally this should be generated from kaffe.def (but isn't).
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>
 */

#ifndef __ops_h
#define __ops_h

/* Information about a Java VM instruction. */
struct {
	/* Control flow change caused by this instruction.
	 * 0:  Not a branch or control flow change.
	 * 1:  Conditional branch.
	 * 2:  A return or athrow.
	 * 3:  A goto or switch.
	 * 4:  A jump (local) subroutine (jsr or jsr_w).
	 */
	char jmp;

	/* Number of bytes taken by this opcode, or 0 if variable. */
	char len;

	/* How much this instruction increases the stack length
	 * (or decreases it if negative).  9 means variable. */
	char stack;

} opcodeInfo[256] = {
    {0,1, 0},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 1},
    {0,1, 1},{0,1, 2},{0,1, 2},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 2},{0,1, 2},
    {0,2, 1},{0,3, 1},{0,2, 1},{0,3, 1},{0,3, 2},{0,2, 1},{0,2, 2},{0,2, 1},
    {0,2, 2},{0,2, 1},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 2},{0,1, 2},
    {0,1, 2},{0,1, 2},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 2},{0,1, 2},
    {0,1, 2},{0,1, 2},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 1},{0,1,-1},{0,1, 0},
    {0,1,-1},{0,1, 0},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,2,-1},{0,2,-2},
    {0,2,-1},{0,2,-2},{0,2,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-2},
    {0,1,-2},{0,1,-2},{0,1,-2},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-2},
    {0,1,-2},{0,1,-2},{0,1,-2},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-3},
    {0,1,-4},{0,1,-3},{0,1,-4},{0,1,-3},{0,1,-3},{0,1,-3},{0,1,-3},{0,1,-1},
    {0,1,-2},{0,1, 1},{0,1, 1},{0,1, 1},{0,1, 2},{0,1, 2},{0,1, 2},{0,1, 0},
    {0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},
    {0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},
    {0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
    {0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-1},{0,1,-2},
    {0,1,-1},{0,1,-2},{0,1,-1},{0,1,-2},{0,3, 0},{0,1, 1},{0,1, 0},{0,1, 1},
    {0,1,-1},{0,1,-1},{0,1, 0},{0,1, 0},{0,1, 1},{0,1, 1},{0,1,-1},{0,1, 0},
    {0,1,-1},{0,1, 0},{0,1, 0},{0,1, 0},{0,1,-3},{0,1,-1},{0,1,-1},{0,1,-3},
    {0,1,-3},{1,3,-1},{1,3,-1},{1,3,-1},{1,3,-1},{1,3,-1},{1,3,-1},{1,3,-2},
    {1,3,-2},{1,3,-2},{1,3,-2},{1,3,-2},{1,3,-2},{1,3,-2},{1,3,-2},{3,3, 0},
    {4,3, 0},{2,2, 0},{3,0,-1},{3,0,-1},{2,1,-1},{2,1,-2},{2,1,-1},{2,1,-2},
    {2,1,-1},{2,1, 0},{0,3, 9},{0,3, 9},{0,3, 9},{0,3, 9},{0,3, 9},{0,3, 9},
    {0,3, 9},{0,5, 9},{0,1, 0},{0,3, 1},{0,2, 0},{0,3, 0},{0,1, 0},{2,1,-1},
    {0,3, 0},{0,3, 0},{0,1,-1},{0,1,-1},{0,1, 0},{0,4, 9},{1,3,-1},{1,3,-1},
    {3,5, 0},{4,5, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
    {0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
    {0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
    {0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
    {0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
    {0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
    {0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},{0,1, 0},
};

#endif
